@@ -1,10 +1,8 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::translate::{from_glib, from_glib_full, IntoGlib, ToGlibPtr};
+use std::{i32, mem, ptr};
 
-use std::i32;
-use std::mem;
-use std::ptr;
+use glib::translate::{from_glib, from_glib_full, IntoGlib, ToGlibPtr};
 
 #[doc(alias = "gst_video_convert_sample")]
 pub fn convert_sample(
@@ -186,8 +184,8 @@ pub fn guess_framerate(duration: gst::ClockTime) -> Option<gst::Fraction> {
         ));
         if res {
             Some(gst::Fraction::new(
-                dest_n.assume_init() as i32,
-                dest_d.assume_init() as i32,
+                dest_n.assume_init(),
+                dest_d.assume_init(),
             ))
         } else {
             None
@@ -195,8 +193,8 @@ pub fn guess_framerate(duration: gst::ClockTime) -> Option<gst::Fraction> {
     }
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 #[doc(alias = "gst_video_is_common_aspect_ratio")]
 pub fn is_common_aspect_ratio(width: u32, height: u32, par: gst::Fraction) -> bool {
     skip_assert_initialized!();
@@ -214,7 +212,7 @@ pub fn is_common_aspect_ratio(width: u32, height: u32, par: gst::Fraction) -> bo
 pub fn video_make_raw_caps(
     formats: &[crate::VideoFormat],
 ) -> crate::VideoCapsBuilder<gst::caps::NoFeature> {
-    assert_initialized_main_thread!();
+    skip_assert_initialized!();
 
     let formats = formats.iter().copied().map(|f| match f {
         crate::VideoFormat::Encoded => panic!("Invalid encoded format"),
@@ -227,9 +225,10 @@ pub fn video_make_raw_caps(
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use std::sync::{Arc, Mutex};
 
+    use super::*;
+
     #[test]
     fn test_convert_sample_async() {
         gst::init().unwrap();
