@@ -1,8 +1,7 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-#![cfg_attr(feature = "dox", feature(doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![allow(clippy::missing_safety_doc)]
-#![allow(clippy::non_send_fields_in_send_ty)]
 #![doc = include_str!("../README.md")]
 
 pub use ffi;
@@ -13,12 +12,7 @@ pub use gst_base;
 macro_rules! assert_initialized_main_thread {
     () => {
         if !gst::INITIALIZED.load(std::sync::atomic::Ordering::SeqCst) {
-            #[allow(unused_unsafe)]
-            if unsafe { gst::ffi::gst_is_initialized() } != glib::ffi::GTRUE {
-                panic!("GStreamer has not been initialized. Call `gst::init` first.");
-            } else {
-                gst::INITIALIZED.store(true, std::sync::atomic::Ordering::SeqCst);
-            }
+            gst::assert_initialized();
         }
     };
 }
@@ -27,26 +21,21 @@ macro_rules! skip_assert_initialized {
     () => {};
 }
 
-#[allow(clippy::unreadable_literal)]
-#[allow(clippy::too_many_arguments)]
-#[allow(clippy::match_same_arms)]
-#[allow(clippy::use_self)]
 #[allow(clippy::needless_borrow)]
-#[allow(unused_imports)]
 mod auto;
 pub use crate::auto::*;
 
+mod enums;
+
 #[cfg(feature = "serde")]
 mod flag_serde;
 
-mod navigation;
-
 mod caps;
 pub use crate::caps::VideoCapsBuilder;
 
 mod caps_features;
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 pub use crate::caps_features::{CAPS_FEATURES_FORMAT_INTERLACED, CAPS_FEATURE_FORMAT_INTERLACED};
 pub use crate::caps_features::{
     CAPS_FEATURES_META_GST_VIDEO_AFFINE_TRANSFORMATION_META,
@@ -57,15 +46,20 @@ pub use crate::caps_features::{
     CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION,
 };
 mod video_color_matrix;
-pub use video_color_matrix::*;
 mod video_format;
 pub use crate::video_format::*;
 mod video_format_info;
 pub use crate::video_format_info::*;
 mod video_info;
 pub use crate::video_info::*;
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+mod video_info_dma_drm;
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+pub use crate::video_info_dma_drm::*;
 pub mod video_frame;
-pub use crate::video_frame::{VideoFrame, VideoFrameRef};
+pub use crate::video_frame::{VideoFrame, VideoFrameExt, VideoFrameRef};
 mod video_overlay;
 pub use crate::video_overlay::is_video_overlay_prepare_window_handle_message;
 
@@ -88,11 +82,11 @@ pub use crate::video_overlay_composition::{
     VideoOverlayRectangleRef,
 };
 pub mod video_meta;
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 pub use crate::video_meta::VideoCaptionMeta;
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 pub use crate::video_meta::{VideoAFDMeta, VideoBarMeta};
 pub use crate::video_meta::{
     VideoAffineTransformationMeta, VideoCropMeta, VideoMeta, VideoOverlayCompositionMeta,
@@ -114,30 +108,52 @@ pub use crate::video_converter::{VideoConverter, VideoConverterConfig};
 mod video_codec_frame;
 mod video_decoder;
 mod video_encoder;
+mod video_filter;
 pub use crate::video_codec_frame::VideoCodecFrame;
 pub mod video_codec_state;
 pub use crate::video_codec_state::{VideoCodecState, VideoCodecStateContext};
 mod utils;
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 mod video_hdr;
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 pub use crate::video_hdr::*;
 
 mod color_balance_channel;
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 mod video_aggregator;
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 mod video_aggregator_convert_pad;
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 mod video_aggregator_pad;
 
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+mod video_vbi;
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+pub use video_vbi::*;
+
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+mod video_vbi_encoder;
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+pub use video_vbi_encoder::*;
+
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+mod video_vbi_parser;
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
+pub use video_vbi_parser::*;
+
 pub const VIDEO_ENCODER_FLOW_NEED_DATA: gst::FlowSuccess = gst::FlowSuccess::CustomSuccess;
 pub const VIDEO_DECODER_FLOW_NEED_DATA: gst::FlowSuccess = gst::FlowSuccess::CustomSuccess;
 
@@ -147,23 +163,22 @@ pub mod prelude {
     #[doc(hidden)]
     pub use gst_base::prelude::*;
 
-    pub use crate::auto::traits::*;
-    pub use crate::navigation::NavigationExtManual;
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     pub use crate::video_aggregator::VideoAggregatorExtManual;
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     pub use crate::video_aggregator_convert_pad::VideoAggregatorConvertPadExtManual;
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     pub use crate::video_aggregator_pad::VideoAggregatorPadExtManual;
-    pub use crate::video_buffer_pool::VideoBufferPoolConfig;
-    pub use crate::video_decoder::VideoDecoderExtManual;
-    pub use crate::video_encoder::VideoEncoderExtManual;
-    pub use crate::video_format::VideoFormatIteratorExt;
-    pub use crate::video_frame::VideoBufferExt;
-    pub use crate::video_overlay::VideoOverlayExtManual;
+    pub use crate::VideoFrameExt;
+    pub use crate::{
+        auto::traits::*, video_buffer_pool::VideoBufferPoolConfig,
+        video_decoder::VideoDecoderExtManual, video_encoder::VideoEncoderExtManual,
+        video_filter::VideoFilterExtManual, video_format::VideoFormatIteratorExt,
+        video_frame::VideoBufferExt, video_overlay::VideoOverlayExtManual,
+    };
 }
 
 pub mod subclass;
