@@ -1,13 +1,13 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
-use crate::{NavigationCommand, NavigationEventType};
-use glib::translate::{from_glib, from_glib_full, IntoGlib, ToGlibPtr};
-use glib::ToSendValue;
-use gst::EventType;
 use std::mem;
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+use glib::{prelude::*, translate::*};
+use gst::EventType;
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 use crate::NavigationModifierType;
+use crate::{NavigationCommand, NavigationEventType};
 
 // FIXME: Copy from gstreamer/src/event.rs
 macro_rules! event_builder_generic_impl {
@@ -26,24 +26,34 @@ macro_rules! event_builder_generic_impl {
             }
         }
 
-        pub fn other_fields(
-            self,
-            other_fields: &[(&'a str, &'a (dyn ToSendValue + Sync))],
-        ) -> Self {
+        pub fn other_field(self, name: &'a str, value: impl ToSendValue) -> Self {
+            let mut other_fields = self.other_fields;
+            other_fields.push((name, value.to_send_value()));
+
             Self {
-                other_fields: self
-                    .other_fields
-                    .iter()
-                    .cloned()
-                    .chain(other_fields.iter().cloned())
-                    .collect(),
+                other_fields,
                 ..self
             }
         }
 
+        #[deprecated = "use build.other_field() instead"]
+        pub fn other_fields(
+            self,
+            other_fields: &[(&'a str, &'a (dyn ToSendValue + Sync))],
+        ) -> Self {
+            let mut s = self;
+
+            for (name, value) in other_fields {
+                s = s.other_field(name, value.to_send_value());
+            }
+
+            s
+        }
+
         #[must_use = "Building the event without using it has no effect"]
+        #[allow(clippy::redundant_closure_call)]
         pub fn build(mut self) -> gst::Event {
-            assert_initialized_main_thread!();
+            skip_assert_initialized!();
             unsafe {
                 let event = $new_fn(&mut self);
                 if let Some(seqnum) = self.seqnum {
@@ -60,7 +70,7 @@ macro_rules! event_builder_generic_impl {
                     );
 
                     for (k, v) in self.other_fields {
-                        s.set_value(k, v.to_send_value());
+                        s.set_value(k, v);
                     }
                 }
 
@@ -74,7 +84,7 @@ macro_rules! event_builder_generic_impl {
 pub struct DownstreamForceKeyUnitEventBuilder<'a> {
     seqnum: Option<gst::Seqnum>,
     running_time_offset: Option<i64>,
-    other_fields: Vec<(&'a str, &'a (dyn ToSendValue + Sync))>,
+    other_fields: Vec<(&'a str, glib::SendValue)>,
     timestamp: Option<gst::ClockTime>,
     stream_time: Option<gst::ClockTime>,
     running_time: Option<gst::ClockTime>,
@@ -151,6 +161,7 @@ pub struct DownstreamForceKeyUnitEvent {
 
 impl DownstreamForceKeyUnitEvent {
     pub fn builder<'a>() -> DownstreamForceKeyUnitEventBuilder<'a> {
+        assert_initialized_main_thread!();
         DownstreamForceKeyUnitEventBuilder::new()
     }
 
@@ -191,7 +202,7 @@ impl DownstreamForceKeyUnitEvent {
 pub struct UpstreamForceKeyUnitEventBuilder<'a> {
     seqnum: Option<gst::Seqnum>,
     running_time_offset: Option<i64>,
-    other_fields: Vec<(&'a str, &'a (dyn ToSendValue + Sync))>,
+    other_fields: Vec<(&'a str, glib::SendValue)>,
     running_time: Option<gst::ClockTime>,
     all_headers: bool,
     count: u32,
@@ -246,6 +257,7 @@ pub struct UpstreamForceKeyUnitEvent {
 
 impl UpstreamForceKeyUnitEvent {
     pub fn builder<'a>() -> UpstreamForceKeyUnitEventBuilder<'a> {
+        assert_initialized_main_thread!();
         UpstreamForceKeyUnitEventBuilder::new()
     }
 
@@ -303,7 +315,7 @@ impl ForceKeyUnitEvent {
 pub struct StillFrameEventBuilder<'a> {
     seqnum: Option<gst::Seqnum>,
     running_time_offset: Option<i64>,
-    other_fields: Vec<(&'a str, &'a (dyn ToSendValue + Sync))>,
+    other_fields: Vec<(&'a str, glib::SendValue)>,
     in_still: bool,
 }
 
@@ -361,24 +373,24 @@ macro_rules! nav_event_builder {
         pub struct $builder<'a> {
             seqnum: Option<gst::Seqnum>,
             running_time_offset: Option<i64>,
-            other_fields: Vec<(&'a str, &'a (dyn ToSendValue + Sync))>,
+            other_fields: Vec<(&'a str, glib::SendValue)>,
             $($field_names: $field_types,)*
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType,
             $($event_field: $event_type,)?
         }
 
         impl<'a> $builder<'a> {
-            pub fn new($($event_field: $event_type)?) -> Self {
+            fn new($($event_field: $event_type)?) -> Self {
                 skip_assert_initialized!();
                 Self {
                     seqnum: None,
                     running_time_offset: None,
                     other_fields: Vec::new(),
                     $($field_names: <$field_types>::default(),)*
-                    #[cfg(any(feature = "v1_22", feature = "dox"))]
-                    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                    #[cfg(feature = "v1_22")]
+                    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                     modifier_state: NavigationModifierType::empty(),
                     $($event_field,)?
                 }
@@ -388,8 +400,8 @@ macro_rules! nav_event_builder {
                 Self { $field_names, ..self }
             })*
 
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             pub fn modifier_state(self, modifier_state: NavigationModifierType) -> Self {
                 Self { modifier_state, ..self }
             }
@@ -412,18 +424,18 @@ nav_event_builder!(
         let event = match s.kind {
             KeyEventType::Press { key } => NavigationEvent::KeyPress {
                 key: key.to_owned(),
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
             KeyEventType::Release { key } => NavigationEvent::KeyRelease {
                 key: key.to_owned(),
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
         };
-        gst::ffi::gst_event_new_navigation(event.structure().to_glib_full())
+        gst::ffi::gst_event_new_navigation(event.structure().into_glib_ptr())
     }
 );
 
@@ -435,8 +447,8 @@ pub enum MouseEventType {
     Release {
         button: i32,
     },
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     Scroll {
         delta_x: f64,
         delta_y: f64,
@@ -452,39 +464,39 @@ nav_event_builder!(
             MouseEventType::Move => NavigationEvent::MouseMove {
                 x: s.x,
                 y: s.y,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
             MouseEventType::Press { button } => NavigationEvent::MouseButtonPress {
                 button,
                 x: s.x,
                 y: s.y,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
             MouseEventType::Release { button } => NavigationEvent::MouseButtonRelease {
                 button,
                 x: s.x,
                 y: s.y,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
-            #[cfg(any(feature = "v1_18", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+            #[cfg(feature = "v1_18")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
             MouseEventType::Scroll { delta_x, delta_y } => NavigationEvent::MouseScroll {
                 x: s.x,
                 y: s.y,
                 delta_x,
                 delta_y,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state: s.modifier_state,
             },
         };
-        gst::ffi::gst_event_new_navigation(event.structure().to_glib_full())
+        gst::ffi::gst_event_new_navigation(event.structure().into_glib_ptr())
     }
 );
 
@@ -492,29 +504,29 @@ nav_event_builder!(
 pub struct CommandEventBuilder<'a> {
     seqnum: Option<gst::Seqnum>,
     running_time_offset: Option<i64>,
-    other_fields: Vec<(&'a str, &'a (dyn ToSendValue + Sync))>,
+    other_fields: Vec<(&'a str, glib::SendValue)>,
     command: NavigationCommand,
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     modifier_state: NavigationModifierType,
 }
 
 impl<'a> CommandEventBuilder<'a> {
-    pub fn new(command: NavigationCommand) -> Self {
+    fn new(command: NavigationCommand) -> Self {
         skip_assert_initialized!();
         Self {
             seqnum: None,
             running_time_offset: None,
             other_fields: Vec::new(),
             command,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn modifier_state(self, modifier_state: NavigationModifierType) -> Self {
         Self {
             modifier_state,
@@ -525,24 +537,24 @@ impl<'a> CommandEventBuilder<'a> {
     event_builder_generic_impl!(|s: &mut Self| {
         let event = NavigationEvent::Command {
             command: s.command,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: s.modifier_state,
         };
-        gst::ffi::gst_event_new_navigation(event.structure().to_glib_full())
+        gst::ffi::gst_event_new_navigation(event.structure().into_glib_ptr())
     });
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 pub enum TouchEventType {
     Down { pressure: f64 },
     Motion { pressure: f64 },
     Up,
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 nav_event_builder!(
     TouchEventBuilder,
     kind: TouchEventType,
@@ -570,19 +582,19 @@ nav_event_builder!(
                 modifier_state: s.modifier_state,
             },
         };
-        gst::ffi::gst_event_new_navigation(event.structure().to_glib_full())
+        gst::ffi::gst_event_new_navigation(event.structure().into_glib_ptr())
     }
 );
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 pub enum TouchMetaEventType {
     Frame,
     Cancel,
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 nav_event_builder!(
     TouchMetaEventBuilder,
     kind: TouchMetaEventType,
@@ -596,7 +608,7 @@ nav_event_builder!(
                 modifier_state: s.modifier_state,
             },
         };
-        gst::ffi::gst_event_new_navigation(event.structure().to_glib_full())
+        gst::ffi::gst_event_new_navigation(event.structure().into_glib_ptr())
     }
 );
 
@@ -607,58 +619,58 @@ const NAVIGATION_EVENT_NAME: &str = "application/x-gst-navigation";
 pub enum NavigationEvent {
     KeyPress {
         key: String,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
     KeyRelease {
         key: String,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
     MouseMove {
         x: f64,
         y: f64,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
     MouseButtonPress {
         button: i32,
         x: f64,
         y: f64,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
     MouseButtonRelease {
         button: i32,
         x: f64,
         y: f64,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
     Command {
         command: NavigationCommand,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     MouseScroll {
         x: f64,
         y: f64,
         delta_x: f64,
         delta_y: f64,
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     TouchDown {
         identifier: u32,
         x: f64,
@@ -666,8 +678,8 @@ pub enum NavigationEvent {
         pressure: f64,
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     TouchMotion {
         identifier: u32,
         x: f64,
@@ -675,21 +687,21 @@ pub enum NavigationEvent {
         pressure: f64,
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     TouchUp {
         identifier: u32,
         x: f64,
         y: f64,
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     TouchFrame {
         modifier_state: NavigationModifierType,
     },
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     TouchCancel {
         modifier_state: NavigationModifierType,
     },
@@ -701,8 +713,8 @@ impl NavigationEvent {
         assert_initialized_main_thread!();
         Self::KeyPress {
             key: key.to_string(),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -712,8 +724,8 @@ impl NavigationEvent {
         assert_initialized_main_thread!();
         Self::KeyRelease {
             key: key.to_string(),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -724,8 +736,8 @@ impl NavigationEvent {
         Self::MouseMove {
             x,
             y,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -737,8 +749,8 @@ impl NavigationEvent {
             button,
             x,
             y,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -750,14 +762,14 @@ impl NavigationEvent {
             button,
             x,
             y,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     #[doc(alias = "gst_navigation_event_new_mouse_scroll")]
     pub fn new_mouse_scroll(x: f64, y: f64, delta_x: f64, delta_y: f64) -> NavigationEvent {
         assert_initialized_main_thread!();
@@ -766,8 +778,8 @@ impl NavigationEvent {
             y,
             delta_x,
             delta_y,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -777,14 +789,14 @@ impl NavigationEvent {
         assert_initialized_main_thread!();
         Self::Command {
             command,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     #[doc(alias = "gst_navigation_event_new_touch_down")]
     pub fn new_touch_down(identifier: u32, x: f64, y: f64, pressure: f64) -> NavigationEvent {
         assert_initialized_main_thread!();
@@ -793,14 +805,14 @@ impl NavigationEvent {
             x,
             y,
             pressure,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     #[doc(alias = "gst_navigation_event_new_touch_motion")]
     pub fn new_touch_motion(identifier: u32, x: f64, y: f64, pressure: f64) -> NavigationEvent {
         assert_initialized_main_thread!();
@@ -809,14 +821,14 @@ impl NavigationEvent {
             x,
             y,
             pressure,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     #[doc(alias = "gst_navigation_event_new_touch_up")]
     pub fn new_touch_up(identifier: u32, x: f64, y: f64) -> NavigationEvent {
         assert_initialized_main_thread!();
@@ -824,32 +836,32 @@ impl NavigationEvent {
             identifier,
             x,
             y,
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     #[doc(alias = "gst_navigation_event_new_touch_frame")]
     pub fn new_touch_frame() -> NavigationEvent {
         assert_initialized_main_thread!();
         Self::TouchFrame {
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     #[doc(alias = "gst_navigation_event_new_touch_cancel")]
     pub fn new_touch_cancel() -> NavigationEvent {
         assert_initialized_main_thread!();
         Self::TouchCancel {
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             modifier_state: NavigationModifierType::empty(),
         }
     }
@@ -883,8 +895,8 @@ impl NavigationEvent {
             .y(y)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     pub fn mouse_scroll_builder(
         x: f64,
         y: f64,
@@ -902,8 +914,8 @@ impl NavigationEvent {
         CommandEventBuilder::new(command)
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn touch_down_builder(
         identifier: u32,
         x: f64,
@@ -917,8 +929,8 @@ impl NavigationEvent {
             .y(y)
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn touch_motion_builder(
         identifier: u32,
         x: f64,
@@ -932,8 +944,8 @@ impl NavigationEvent {
             .y(y)
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn touch_up_builder(identifier: u32, x: f64, y: f64) -> TouchEventBuilder<'static> {
         assert_initialized_main_thread!();
         TouchEventBuilder::new(TouchEventType::Up)
@@ -942,15 +954,15 @@ impl NavigationEvent {
             .y(y)
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn touch_frame_builder() -> TouchMetaEventBuilder<'static> {
         assert_initialized_main_thread!();
         TouchMetaEventBuilder::new(TouchMetaEventType::Frame)
     }
 
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     pub fn touch_cancel_builder() -> TouchMetaEventBuilder<'static> {
         assert_initialized_main_thread!();
         TouchMetaEventBuilder::new(TouchMetaEventType::Cancel)
@@ -958,7 +970,7 @@ impl NavigationEvent {
 
     #[doc(alias = "gst_navigation_event_get_type")]
     pub fn type_(event: &gst::EventRef) -> NavigationEventType {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         unsafe { from_glib(ffi::gst_navigation_event_get_type(event.as_mut_ptr())) }
     }
 
@@ -982,8 +994,8 @@ impl NavigationEvent {
             return Err(glib::bool_error!("Invalid navigation event"));
         }
 
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+        #[cfg(feature = "v1_22")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
         let modifier_state = structure
             .get("state")
             .unwrap_or(NavigationModifierType::empty());
@@ -995,8 +1007,8 @@ impl NavigationEvent {
                 y: structure
                     .get("pointer_y")
                     .map_err(|_| glib::bool_error!("Invalid mouse event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
             NavigationEventType::MouseButtonPress => NavigationEvent::MouseButtonPress {
@@ -1009,8 +1021,8 @@ impl NavigationEvent {
                 y: structure
                     .get("pointer_y")
                     .map_err(|_| glib::bool_error!("Invalid mouse event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
             NavigationEventType::MouseButtonRelease => NavigationEvent::MouseButtonRelease {
@@ -1023,12 +1035,12 @@ impl NavigationEvent {
                 y: structure
                     .get("pointer_y")
                     .map_err(|_| glib::bool_error!("Invalid mouse event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
-            #[cfg(any(feature = "v1_18", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+            #[cfg(feature = "v1_18")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
             NavigationEventType::MouseScroll => NavigationEvent::MouseScroll {
                 x: structure
                     .get("pointer_x")
@@ -1042,36 +1054,36 @@ impl NavigationEvent {
                 delta_y: structure
                     .get("delta_pointer_y")
                     .map_err(|_| glib::bool_error!("Invalid mouse event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
             NavigationEventType::KeyPress => NavigationEvent::KeyPress {
                 key: structure
                     .get("key")
                     .map_err(|_| glib::bool_error!("Invalid key press event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
             NavigationEventType::KeyRelease => NavigationEvent::KeyRelease {
                 key: structure
                     .get("key")
                     .map_err(|_| glib::bool_error!("Invalid key press event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
             NavigationEventType::Command => NavigationEvent::Command {
                 command: structure
                     .get("command-code")
                     .map_err(|_| glib::bool_error!("Invalid key press event"))?,
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
-                #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+                #[cfg(feature = "v1_22")]
+                #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
                 modifier_state,
             },
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             NavigationEventType::TouchDown => NavigationEvent::TouchDown {
                 identifier: structure
                     .get("identifier")
@@ -1087,8 +1099,8 @@ impl NavigationEvent {
                     .map_err(|_| glib::bool_error!("Invalid touch event"))?,
                 modifier_state,
             },
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             NavigationEventType::TouchMotion => NavigationEvent::TouchMotion {
                 identifier: structure
                     .get("identifier")
@@ -1104,8 +1116,8 @@ impl NavigationEvent {
                     .map_err(|_| glib::bool_error!("Invalid touch event"))?,
                 modifier_state,
             },
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             NavigationEventType::TouchUp => NavigationEvent::TouchUp {
                 identifier: structure
                     .get("identifier")
@@ -1118,11 +1130,11 @@ impl NavigationEvent {
                     .map_err(|_| glib::bool_error!("Invalid touch event"))?,
                 modifier_state,
             },
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             NavigationEventType::TouchFrame => NavigationEvent::TouchFrame { modifier_state },
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             NavigationEventType::TouchCancel => NavigationEvent::TouchCancel { modifier_state },
             NavigationEventType::Invalid | NavigationEventType::__Unknown(_) => {
                 return Err(glib::bool_error!("Invalid navigation event"))
@@ -1153,8 +1165,8 @@ impl NavigationEvent {
                     .field("pointer_x", x)
                     .field("pointer_y", y)
             }
-            #[cfg(any(feature = "v1_18", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+            #[cfg(feature = "v1_18")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
             Self::MouseScroll {
                 x,
                 y,
@@ -1176,8 +1188,8 @@ impl NavigationEvent {
             Self::Command { command, .. } => gst::Structure::builder(NAVIGATION_EVENT_NAME)
                 .field("event", "command")
                 .field("command-code", command),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             Self::TouchDown {
                 identifier,
                 x,
@@ -1190,8 +1202,8 @@ impl NavigationEvent {
                 .field("pointer_x", x)
                 .field("pointer_y", y)
                 .field("pressure", pressure),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             Self::TouchMotion {
                 identifier,
                 x,
@@ -1204,8 +1216,8 @@ impl NavigationEvent {
                 .field("pointer_x", x)
                 .field("pointer_y", y)
                 .field("pressure", pressure),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             Self::TouchUp {
                 identifier, x, y, ..
             } => gst::Structure::builder(NAVIGATION_EVENT_NAME)
@@ -1213,19 +1225,19 @@ impl NavigationEvent {
                 .field("identifier", identifier)
                 .field("pointer_x", x)
                 .field("pointer_y", y),
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             Self::TouchFrame { .. } => {
                 gst::Structure::builder(NAVIGATION_EVENT_NAME).field("event", "touch-frame")
             }
-            #[cfg(any(feature = "v1_22", feature = "dox"))]
-            #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+            #[cfg(feature = "v1_22")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
             Self::TouchCancel { .. } => {
                 gst::Structure::builder(NAVIGATION_EVENT_NAME).field("event", "touch-cancel")
             }
         };
 
-        #[cfg(any(feature = "v1_22", feature = "dox"))]
+        #[cfg(feature = "v1_22")]
         if true {
             structure = match self {
                 Self::MouseMove { modifier_state, .. } => structure.field("state", modifier_state),
@@ -1267,8 +1279,8 @@ impl NavigationEvent {
 mod tests {
     #[test]
     #[cfg(feature = "serde")]
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     fn serialize_navigation_events() {
         use crate::{NavigationEvent, NavigationModifierType};
 
