@@ -1,10 +1,9 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use once_cell::sync::Lazy;
-use std::ffi::CStr;
 use std::str;
 
 use glib::translate::{from_glib, FromGlib, IntoGlib};
+use once_cell::sync::Lazy;
 
 #[cfg(feature = "v1_18")]
 pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| unsafe {
@@ -40,13 +39,13 @@ pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| {
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Y410,
             crate::VideoFormat::Gbra,
-            crate::VideoFormat::Abgr,
+            crate::VideoFormat::Ayuv,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Vuya,
-            crate::VideoFormat::Bgra,
-            crate::VideoFormat::Ayuv,
-            crate::VideoFormat::Argb,
             crate::VideoFormat::Rgba,
+            crate::VideoFormat::Argb,
+            crate::VideoFormat::Bgra,
+            crate::VideoFormat::Abgr,
             crate::VideoFormat::A420,
             crate::VideoFormat::V216,
             crate::VideoFormat::Y44412le,
@@ -67,46 +66,46 @@ pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| {
             crate::VideoFormat::Nv1610le32,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Y210,
-            crate::VideoFormat::V210,
             crate::VideoFormat::Uyvp,
+            crate::VideoFormat::V210,
             crate::VideoFormat::I42010le,
             crate::VideoFormat::I42010be,
             crate::VideoFormat::P01010le,
-            crate::VideoFormat::Nv1210le32,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Nv1210le40,
+            crate::VideoFormat::Nv1210le32,
             crate::VideoFormat::P01010be,
             crate::VideoFormat::Y444,
             crate::VideoFormat::Gbr,
             crate::VideoFormat::Nv24,
-            crate::VideoFormat::Xbgr,
-            crate::VideoFormat::Bgrx,
-            crate::VideoFormat::Xrgb,
-            crate::VideoFormat::Rgbx,
-            crate::VideoFormat::Bgr,
-            crate::VideoFormat::Iyu2,
             crate::VideoFormat::V308,
+            crate::VideoFormat::Iyu2,
+            crate::VideoFormat::Rgbx,
+            crate::VideoFormat::Xrgb,
+            crate::VideoFormat::Bgrx,
+            crate::VideoFormat::Xbgr,
             crate::VideoFormat::Rgb,
+            crate::VideoFormat::Bgr,
             crate::VideoFormat::Y42b,
-            crate::VideoFormat::Nv61,
             crate::VideoFormat::Nv16,
-            crate::VideoFormat::Vyuy,
-            crate::VideoFormat::Uyvy,
-            crate::VideoFormat::Yvyu,
+            crate::VideoFormat::Nv61,
             crate::VideoFormat::Yuy2,
+            crate::VideoFormat::Yvyu,
+            crate::VideoFormat::Uyvy,
+            crate::VideoFormat::Vyuy,
             crate::VideoFormat::I420,
             crate::VideoFormat::Yv12,
-            crate::VideoFormat::Nv21,
             crate::VideoFormat::Nv12,
+            crate::VideoFormat::Nv21,
             crate::VideoFormat::Nv1264z32,
             crate::VideoFormat::Y41b,
             crate::VideoFormat::Iyu1,
-            crate::VideoFormat::Yvu9,
             crate::VideoFormat::Yuv9,
-            crate::VideoFormat::Rgb16,
+            crate::VideoFormat::Yvu9,
             crate::VideoFormat::Bgr16,
-            crate::VideoFormat::Rgb15,
+            crate::VideoFormat::Rgb16,
             crate::VideoFormat::Bgr15,
+            crate::VideoFormat::Rgb15,
             crate::VideoFormat::Rgb8p,
             crate::VideoFormat::Gray16Le,
             crate::VideoFormat::Gray16Be,
@@ -130,17 +129,17 @@ pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| {
             crate::VideoFormat::A42010be,
             crate::VideoFormat::A42010le,
             #[cfg(feature = "v1_16")]
-            crate::VideoFormat::Y410,
-            #[cfg(feature = "v1_16")]
             crate::VideoFormat::Bgr10a2Le,
+            #[cfg(feature = "v1_16")]
+            crate::VideoFormat::Y410,
             crate::VideoFormat::Gbra,
-            crate::VideoFormat::Abgr,
+            crate::VideoFormat::Ayuv,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Vuya,
-            crate::VideoFormat::Bgra,
-            crate::VideoFormat::Ayuv,
-            crate::VideoFormat::Argb,
             crate::VideoFormat::Rgba,
+            crate::VideoFormat::Argb,
+            crate::VideoFormat::Bgra,
+            crate::VideoFormat::Abgr,
             crate::VideoFormat::A420,
             crate::VideoFormat::V216,
             crate::VideoFormat::Y44412be,
@@ -161,46 +160,46 @@ pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| {
             crate::VideoFormat::Nv1610le32,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Y210,
-            crate::VideoFormat::V210,
             crate::VideoFormat::Uyvp,
+            crate::VideoFormat::V210,
             crate::VideoFormat::I42010be,
             crate::VideoFormat::I42010le,
             crate::VideoFormat::P01010be,
             crate::VideoFormat::P01010le,
-            crate::VideoFormat::Nv1210le32,
             #[cfg(feature = "v1_16")]
             crate::VideoFormat::Nv1210le40,
+            crate::VideoFormat::Nv1210le32,
             crate::VideoFormat::Y444,
             crate::VideoFormat::Gbr,
             crate::VideoFormat::Nv24,
-            crate::VideoFormat::Xbgr,
-            crate::VideoFormat::Bgrx,
-            crate::VideoFormat::Xrgb,
-            crate::VideoFormat::Rgbx,
-            crate::VideoFormat::Bgr,
-            crate::VideoFormat::Iyu2,
             crate::VideoFormat::V308,
+            crate::VideoFormat::Iyu2,
+            crate::VideoFormat::Rgbx,
+            crate::VideoFormat::Xrgb,
+            crate::VideoFormat::Bgrx,
+            crate::VideoFormat::Xbgr,
             crate::VideoFormat::Rgb,
+            crate::VideoFormat::Bgr,
             crate::VideoFormat::Y42b,
-            crate::VideoFormat::Nv61,
             crate::VideoFormat::Nv16,
-            crate::VideoFormat::Vyuy,
-            crate::VideoFormat::Uyvy,
-            crate::VideoFormat::Yvyu,
+            crate::VideoFormat::Nv61,
             crate::VideoFormat::Yuy2,
+            crate::VideoFormat::Yvyu,
+            crate::VideoFormat::Uyvy,
+            crate::VideoFormat::Vyuy,
             crate::VideoFormat::I420,
             crate::VideoFormat::Yv12,
-            crate::VideoFormat::Nv21,
             crate::VideoFormat::Nv12,
+            crate::VideoFormat::Nv21,
             crate::VideoFormat::Nv1264z32,
             crate::VideoFormat::Y41b,
             crate::VideoFormat::Iyu1,
-            crate::VideoFormat::Yvu9,
             crate::VideoFormat::Yuv9,
-            crate::VideoFormat::Rgb16,
+            crate::VideoFormat::Yvu9,
             crate::VideoFormat::Bgr16,
-            crate::VideoFormat::Rgb15,
+            crate::VideoFormat::Rgb16,
             crate::VideoFormat::Bgr15,
+            crate::VideoFormat::Rgb15,
             crate::VideoFormat::Rgb8p,
             crate::VideoFormat::Gray16Be,
             crate::VideoFormat::Gray16Le,
@@ -210,6 +209,18 @@ pub static VIDEO_FORMATS_ALL: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| {
     }
 });
 
+#[cfg(feature = "v1_24")]
+pub static VIDEO_FORMATS_ANY: Lazy<Box<[crate::VideoFormat]>> = Lazy::new(|| unsafe {
+    let mut len: u32 = 0;
+    let mut res = Vec::with_capacity(len as usize);
+    let formats = ffi::gst_video_formats_any(&mut len);
+    for i in 0..len {
+        let format = formats.offset(i as isize);
+        res.push(from_glib(*format));
+    }
+    res.into_boxed_slice()
+});
+
 #[derive(PartialEq, Eq, Copy, Clone, Debug, Hash)]
 pub enum VideoEndianness {
     Unknown,
@@ -218,6 +229,7 @@ pub enum VideoEndianness {
 }
 
 impl FromGlib<i32> for VideoEndianness {
+    #[inline]
     unsafe fn from_glib(value: i32) -> Self {
         skip_assert_initialized!();
 
@@ -232,6 +244,7 @@ impl FromGlib<i32> for VideoEndianness {
 impl IntoGlib for VideoEndianness {
     type GlibType = i32;
 
+    #[inline]
     fn into_glib(self) -> i32 {
         match self {
             Self::LittleEndian => 1234,
@@ -268,24 +281,27 @@ impl crate::VideoFormat {
     }
 
     #[doc(alias = "gst_video_format_to_string")]
-    pub fn to_str<'a>(self) -> &'a str {
+    pub fn to_str<'a>(self) -> &'a glib::GStr {
         if self == Self::Unknown {
-            return "UNKNOWN";
+            return glib::gstr!("UNKNOWN");
         }
         unsafe {
-            CStr::from_ptr(
+            glib::GStr::from_ptr(
                 ffi::gst_video_format_to_string(self.into_glib())
                     .as_ref()
                     .expect("gst_video_format_to_string returned NULL"),
             )
-            .to_str()
-            .expect("gst_video_format_to_string returned an invalid string")
         }
     }
 
     pub fn iter_raw() -> VideoFormatIterator {
         VideoFormatIterator::default()
     }
+
+    #[cfg(feature = "v1_24")]
+    pub fn iter_any() -> impl Iterator<Item = crate::VideoFormat> {
+        VIDEO_FORMATS_ANY.iter().copied()
+    }
 }
 
 impl str::FromStr for crate::VideoFormat {
@@ -306,13 +322,14 @@ impl str::FromStr for crate::VideoFormat {
 }
 
 impl PartialOrd for crate::VideoFormat {
+    #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        crate::VideoFormatInfo::from_format(*self)
-            .partial_cmp(&crate::VideoFormatInfo::from_format(*other))
+        Some(self.cmp(other))
     }
 }
 
 impl Ord for crate::VideoFormat {
+    #[inline]
     fn cmp(&self, other: &Self) -> std::cmp::Ordering {
         crate::VideoFormatInfo::from_format(*self).cmp(&crate::VideoFormatInfo::from_format(*other))
     }
@@ -350,7 +367,7 @@ impl Iterator for VideoFormatIterator {
             return (0, Some(0));
         }
 
-        let remaining = (self.len - self.idx) as usize;
+        let remaining = self.len - self.idx;
 
         (remaining, Some(remaining))
     }
@@ -524,7 +541,6 @@ mod tests {
         assert_eq!(caps.to_string(), "video/x-raw, format=(string){ NV12, NV16 }, width=(int)[ 1, 2147483647 ], height=(int)[ 1, 2147483647 ], framerate=(fraction)[ 0/1, 2147483647/1 ]");
     }
 
-    #[cfg(feature = "v1_18")]
     #[test]
     fn sort() {
         use itertools::Itertools;
@@ -536,6 +552,8 @@ mod tests {
                 > crate::VideoFormatInfo::from_format(crate::VideoFormat::Nv12)
         );
         assert!(crate::VideoFormat::I420 > crate::VideoFormat::Yv12);
+        assert!(crate::VideoFormat::Nv12 > crate::VideoFormat::Nv21);
+        assert!(crate::VideoFormat::Xrgb > crate::VideoFormat::Rgb);
 
         let sorted: Vec<crate::VideoFormat> =
             crate::VideoFormat::iter_raw().sorted().rev().collect();
