@@ -1,21 +1,26 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::translate::{from_glib, from_glib_none, Borrowed, ToGlibPtr};
+use std::{fmt, marker::PhantomData, mem, ops, ptr, slice};
 
-use std::fmt;
-use std::marker::PhantomData;
-use std::mem;
-use std::ops;
-use std::ptr;
-use std::slice;
+use glib::translate::{from_glib, from_glib_none, Borrowed, ToGlibPtr};
 
 pub enum Readable {}
 pub enum Writable {}
 
+pub trait IsVideoFrame {
+    fn as_raw(&self) -> &ffi::GstVideoFrame;
+}
+
+impl<T> IsVideoFrame for VideoFrame<T> {
+    #[inline]
+    fn as_raw(&self) -> &ffi::GstVideoFrame {
+        &self.frame
+    }
+}
+
 pub struct VideoFrame<T> {
     frame: ffi::GstVideoFrame,
-    buffer: Option<gst::Buffer>,
-    info: crate::VideoInfo,
+    buffer: gst::Buffer,
     phantom: PhantomData<T>,
 }
 
@@ -25,29 +30,178 @@ unsafe impl<T> Sync for VideoFrame<T> {}
 impl<T> fmt::Debug for VideoFrame<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("VideoFrame")
-            .field("frame", &self.frame)
-            .field("buffer", &self.buffer)
-            .field("info", &self.info)
-            .field("phantom", &self.phantom)
+            .field("flags", &self.flags())
+            .field("id", &self.id())
+            .field("buffer", &self.buffer())
+            .field("info", &self.info())
             .finish()
     }
 }
 
+mod sealed {
+    pub trait Sealed {}
+    impl<T: super::IsVideoFrame> Sealed for T {}
+}
+
+pub trait VideoFrameExt: sealed::Sealed + IsVideoFrame {
+    #[inline]
+    fn as_ptr(&self) -> *const ffi::GstVideoFrame {
+        self.as_raw() as _
+    }
+
+    #[inline]
+    fn info(&self) -> &crate::VideoInfo {
+        unsafe {
+            let frame = self.as_raw();
+            let info = &frame.info as *const ffi::GstVideoInfo as *const crate::VideoInfo;
+            &*info
+        }
+    }
+
+    #[inline]
+    fn flags(&self) -> crate::VideoFrameFlags {
+        unsafe { from_glib(self.as_raw().flags) }
+    }
+
+    #[inline]
+    fn id(&self) -> i32 {
+        self.as_raw().id
+    }
+
+    #[inline]
+    fn buffer(&self) -> &gst::BufferRef {
+        unsafe { gst::BufferRef::from_ptr(self.as_raw().buffer) }
+    }
+
+    #[inline]
+    fn format(&self) -> crate::VideoFormat {
+        self.info().format()
+    }
+
+    #[inline]
+    fn format_info(&self) -> crate::VideoFormatInfo {
+        self.info().format_info()
+    }
+
+    #[inline]
+    fn width(&self) -> u32 {
+        self.info().width()
+    }
+
+    #[inline]
+    fn height(&self) -> u32 {
+        self.info().height()
+    }
+
+    #[inline]
+    fn size(&self) -> usize {
+        self.info().size()
+    }
+
+    #[inline]
+    fn is_interlaced(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::INTERLACED)
+    }
+
+    #[inline]
+    fn is_tff(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::TFF)
+    }
+
+    #[inline]
+    fn is_rff(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::RFF)
+    }
+
+    #[inline]
+    fn is_onefield(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
+    }
+
+    #[inline]
+    fn is_bottom_field(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
+            && !self.flags().contains(crate::VideoFrameFlags::TFF)
+    }
+
+    #[inline]
+    fn is_top_field(&self) -> bool {
+        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
+            && self.flags().contains(crate::VideoFrameFlags::TFF)
+    }
+
+    #[inline]
+    fn n_planes(&self) -> u32 {
+        self.info().n_planes()
+    }
+
+    #[inline]
+    fn n_components(&self) -> u32 {
+        self.info().n_components()
+    }
+
+    #[inline]
+    fn plane_stride(&self) -> &[i32] {
+        self.info().stride()
+    }
+
+    #[inline]
+    fn plane_offset(&self) -> &[usize] {
+        self.info().offset()
+    }
+
+    #[inline]
+    fn comp_depth(&self, component: u32) -> u32 {
+        self.info().comp_depth(component as u8)
+    }
+
+    #[inline]
+    fn comp_height(&self, component: u32) -> u32 {
+        self.info().comp_height(component as u8)
+    }
+
+    #[inline]
+    fn comp_width(&self, component: u32) -> u32 {
+        self.info().comp_width(component as u8)
+    }
+
+    #[inline]
+    fn comp_offset(&self, component: u32) -> usize {
+        self.info().comp_offset(component as u8)
+    }
+
+    #[inline]
+    fn comp_poffset(&self, component: u32) -> u32 {
+        self.info().comp_poffset(component as u8)
+    }
+
+    #[inline]
+    fn comp_pstride(&self, component: u32) -> i32 {
+        self.info().comp_pstride(component as u8)
+    }
+
+    #[inline]
+    fn comp_stride(&self, component: u32) -> i32 {
+        self.info().comp_stride(component as u8)
+    }
+
+    #[inline]
+    fn comp_plane(&self, component: u32) -> u32 {
+        self.info().comp_plane(component as u8)
+    }
+}
+
+impl<O: IsVideoFrame> VideoFrameExt for O {}
+
 impl<T> VideoFrame<T> {
-    pub fn info(&self) -> &crate::VideoInfo {
-        &self.info
-    }
-
-    pub fn flags(&self) -> crate::VideoFrameFlags {
-        unsafe { from_glib(self.frame.flags) }
-    }
-
-    pub fn id(&self) -> i32 {
-        self.frame.id
-    }
-
-    pub fn into_buffer(mut self) -> gst::Buffer {
-        self.buffer.take().unwrap()
+    #[inline]
+    pub fn into_buffer(self) -> gst::Buffer {
+        unsafe {
+            let mut s = mem::ManuallyDrop::new(self);
+            let buffer = ptr::read(&s.buffer);
+            ffi::gst_video_frame_unmap(&mut s.frame);
+            buffer
+        }
     }
 
     #[doc(alias = "gst_video_frame_copy")]
@@ -84,105 +238,13 @@ impl<T> VideoFrame<T> {
         }
     }
 
-    pub fn format(&self) -> crate::VideoFormat {
-        self.info().format()
-    }
-
-    pub fn format_info(&self) -> crate::VideoFormatInfo {
-        self.info().format_info()
-    }
-
-    pub fn width(&self) -> u32 {
-        self.info().width()
-    }
-
-    pub fn height(&self) -> u32 {
-        self.info().height()
-    }
-
-    pub fn size(&self) -> usize {
-        self.info().size()
-    }
-
-    pub fn is_interlaced(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::INTERLACED)
-    }
-
-    pub fn is_tff(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn is_rff(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::RFF)
-    }
-
-    pub fn is_onefield(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-    }
-
-    pub fn is_bottom_field(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-            && !self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn is_top_field(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-            && self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn n_planes(&self) -> u32 {
-        self.info().n_planes()
-    }
-
-    pub fn n_components(&self) -> u32 {
-        self.info().n_components()
-    }
-
-    pub fn plane_stride(&self) -> &[i32] {
-        self.info().stride()
-    }
-
-    pub fn plane_offset(&self) -> &[usize] {
-        self.info().offset()
-    }
-
+    #[inline]
     pub fn comp_data(&self, component: u32) -> Result<&[u8], glib::BoolError> {
         let poffset = self.info().comp_poffset(component as u8) as usize;
         Ok(&self.plane_data(self.format_info().plane()[component as usize])?[poffset..])
     }
 
-    pub fn comp_depth(&self, component: u32) -> u32 {
-        self.info().comp_depth(component as u8)
-    }
-
-    pub fn comp_height(&self, component: u32) -> u32 {
-        self.info().comp_height(component as u8)
-    }
-
-    pub fn comp_width(&self, component: u32) -> u32 {
-        self.info().comp_width(component as u8)
-    }
-
-    pub fn comp_offset(&self, component: u32) -> usize {
-        self.info().comp_offset(component as u8)
-    }
-
-    pub fn comp_poffset(&self, component: u32) -> u32 {
-        self.info().comp_poffset(component as u8)
-    }
-
-    pub fn comp_pstride(&self, component: u32) -> i32 {
-        self.info().comp_pstride(component as u8)
-    }
-
-    pub fn comp_stride(&self, component: u32) -> i32 {
-        self.info().comp_stride(component as u8)
-    }
-
-    pub fn comp_plane(&self, component: u32) -> u32 {
-        self.info().comp_plane(component as u8)
-    }
-
+    #[inline]
     pub fn buffer(&self) -> &gst::BufferRef {
         unsafe { gst::BufferRef::from_ptr(self.frame.buffer) }
     }
@@ -223,40 +285,48 @@ impl<T> VideoFrame<T> {
         }
     }
 
+    pub fn planes_data(&self) -> [&[u8]; 4] {
+        let mut planes = [[].as_slice(); 4];
+
+        for plane in 0..self.n_planes() {
+            planes[plane as usize] = self.plane_data(plane).unwrap();
+        }
+
+        planes
+    }
+
+    #[inline]
     pub unsafe fn from_glib_full(frame: ffi::GstVideoFrame) -> Self {
-        let info = crate::VideoInfo(ptr::read(&frame.info));
         let buffer = gst::Buffer::from_glib_none(frame.buffer);
         Self {
             frame,
-            buffer: Some(buffer),
-            info,
+            buffer,
             phantom: PhantomData,
         }
     }
 
+    #[inline]
     pub fn as_video_frame_ref(&self) -> VideoFrameRef<&gst::BufferRef> {
         let frame = unsafe { ptr::read(&self.frame) };
-        let info = self.info.clone();
         VideoFrameRef {
             frame,
-            buffer: Some(self.buffer()),
-            info,
             unmap: false,
+            phantom: PhantomData,
         }
     }
 
-    pub fn as_ptr(&self) -> *const ffi::GstVideoFrame {
-        &self.frame
-    }
-
+    #[inline]
     pub fn into_raw(self) -> ffi::GstVideoFrame {
-        let mut s = mem::ManuallyDrop::new(self);
-        s.buffer = None;
-        s.frame
+        unsafe {
+            let mut s = mem::ManuallyDrop::new(self);
+            ptr::drop_in_place(&mut s.buffer);
+            s.frame
+        }
     }
 }
 
 impl<T> Drop for VideoFrame<T> {
+    #[inline]
     fn drop(&mut self) {
         unsafe {
             ffi::gst_video_frame_unmap(&mut self.frame);
@@ -265,6 +335,7 @@ impl<T> Drop for VideoFrame<T> {
 }
 
 impl VideoFrame<Readable> {
+    #[inline]
     pub fn from_buffer_readable(
         buffer: gst::Buffer,
         info: &crate::VideoInfo,
@@ -274,7 +345,7 @@ impl VideoFrame<Readable> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -286,17 +357,16 @@ impl VideoFrame<Readable> {
                 Err(buffer)
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
+                    buffer,
                     phantom: PhantomData,
                 })
             }
         }
     }
 
+    #[inline]
     pub fn from_buffer_id_readable(
         buffer: gst::Buffer,
         id: i32,
@@ -307,7 +377,7 @@ impl VideoFrame<Readable> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map_id(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -320,23 +390,23 @@ impl VideoFrame<Readable> {
                 Err(buffer)
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
+                    buffer,
                     phantom: PhantomData,
                 })
             }
         }
     }
 
+    #[inline]
     pub fn buffer_owned(&self) -> gst::Buffer {
         unsafe { from_glib_none(self.frame.buffer) }
     }
 }
 
 impl VideoFrame<Writable> {
+    #[inline]
     pub fn from_buffer_writable(
         buffer: gst::Buffer,
         info: &crate::VideoInfo,
@@ -346,7 +416,7 @@ impl VideoFrame<Writable> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -360,17 +430,16 @@ impl VideoFrame<Writable> {
                 Err(buffer)
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
+                    buffer,
                     phantom: PhantomData,
                 })
             }
         }
     }
 
+    #[inline]
     pub fn from_buffer_id_writable(
         buffer: gst::Buffer,
         id: i32,
@@ -381,7 +450,7 @@ impl VideoFrame<Writable> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map_id(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -396,21 +465,15 @@ impl VideoFrame<Writable> {
                 Err(buffer)
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
+                    buffer,
                     phantom: PhantomData,
                 })
             }
         }
     }
 
-    pub fn buffer_mut(&mut self) -> &mut gst::BufferRef {
-        unsafe { gst::BufferRef::from_mut_ptr(self.frame.buffer) }
-    }
-
     pub fn comp_data_mut(&mut self, component: u32) -> Result<&mut [u8], glib::BoolError> {
         let poffset = self.info().comp_poffset(component as u8) as usize;
         Ok(&mut self.plane_data_mut(self.format_info().plane()[component as usize])?[poffset..])
@@ -452,43 +515,67 @@ impl VideoFrame<Writable> {
         }
     }
 
-    pub fn as_mut_video_frame_ref(&mut self) -> VideoFrameRef<&mut gst::BufferRef> {
-        let frame = unsafe { ptr::read(&self.frame) };
-        let info = self.info.clone();
-        VideoFrameRef {
-            frame,
-            buffer: Some(self.buffer_mut()),
-            info,
-            unmap: false,
+    pub fn planes_data_mut(&mut self) -> [&mut [u8]; 4] {
+        unsafe {
+            let mut planes = [
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+            ];
+
+            for plane in 0..self.n_planes() {
+                let slice = self.plane_data_mut(plane).unwrap();
+                planes[plane as usize] = slice::from_raw_parts_mut(slice.as_mut_ptr(), slice.len());
+            }
+
+            planes
         }
     }
 
+    #[inline]
+    pub fn as_mut_video_frame_ref(&mut self) -> VideoFrameRef<&mut gst::BufferRef> {
+        let frame = unsafe { ptr::read(&self.frame) };
+        VideoFrameRef {
+            frame,
+            unmap: false,
+            phantom: PhantomData,
+        }
+    }
+
+    #[inline]
     pub fn as_mut_ptr(&mut self) -> *mut ffi::GstVideoFrame {
         &mut self.frame
     }
 }
 
-#[derive(Debug)]
 pub struct VideoFrameRef<T> {
     frame: ffi::GstVideoFrame,
-    buffer: Option<T>,
-    info: crate::VideoInfo,
     unmap: bool,
+    phantom: PhantomData<T>,
+}
+
+impl<T> IsVideoFrame for VideoFrameRef<T> {
+    #[inline]
+    fn as_raw(&self) -> &ffi::GstVideoFrame {
+        &self.frame
+    }
+}
+
+impl<T> fmt::Debug for VideoFrameRef<T> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("VideoFrameRef")
+            .field("flags", &self.flags())
+            .field("id", &self.id())
+            .field("buffer", &unsafe {
+                gst::BufferRef::from_ptr(self.frame.buffer)
+            })
+            .field("info", &self.info())
+            .finish()
+    }
 }
 
 impl<T> VideoFrameRef<T> {
-    pub fn info(&self) -> &crate::VideoInfo {
-        &self.info
-    }
-
-    pub fn flags(&self) -> crate::VideoFrameFlags {
-        unsafe { from_glib(self.frame.flags) }
-    }
-
-    pub fn id(&self) -> i32 {
-        self.frame.id
-    }
-
     #[doc(alias = "gst_video_frame_copy")]
     pub fn copy(
         &self,
@@ -526,105 +613,11 @@ impl<T> VideoFrameRef<T> {
         }
     }
 
-    pub fn format(&self) -> crate::VideoFormat {
-        self.info().format()
-    }
-
-    pub fn format_info(&self) -> crate::VideoFormatInfo {
-        self.info().format_info()
-    }
-
-    pub fn width(&self) -> u32 {
-        self.info().width()
-    }
-
-    pub fn height(&self) -> u32 {
-        self.info().height()
-    }
-
-    pub fn size(&self) -> usize {
-        self.info().size()
-    }
-
-    pub fn is_interlaced(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::INTERLACED)
-    }
-
-    pub fn is_tff(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn is_rff(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::RFF)
-    }
-
-    pub fn is_onefield(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-    }
-
-    pub fn is_bottom_field(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-            && !self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn is_top_field(&self) -> bool {
-        self.flags().contains(crate::VideoFrameFlags::ONEFIELD)
-            && self.flags().contains(crate::VideoFrameFlags::TFF)
-    }
-
-    pub fn n_planes(&self) -> u32 {
-        self.info().n_planes()
-    }
-
-    pub fn n_components(&self) -> u32 {
-        self.info().n_components()
-    }
-
-    pub fn plane_stride(&self) -> &[i32] {
-        self.info().stride()
-    }
-
-    pub fn plane_offset(&self) -> &[usize] {
-        self.info().offset()
-    }
-
     pub fn comp_data(&self, component: u32) -> Result<&[u8], glib::BoolError> {
         let poffset = self.info().comp_poffset(component as u8) as usize;
         Ok(&self.plane_data(self.format_info().plane()[component as usize])?[poffset..])
     }
 
-    pub fn comp_depth(&self, component: u32) -> u32 {
-        self.info().comp_depth(component as u8)
-    }
-
-    pub fn comp_height(&self, component: u32) -> u32 {
-        self.info().comp_height(component as u8)
-    }
-
-    pub fn comp_width(&self, component: u32) -> u32 {
-        self.info().comp_width(component as u8)
-    }
-
-    pub fn comp_offset(&self, component: u32) -> usize {
-        self.info().comp_offset(component as u8)
-    }
-
-    pub fn comp_poffset(&self, component: u32) -> u32 {
-        self.info().comp_poffset(component as u8)
-    }
-
-    pub fn comp_pstride(&self, component: u32) -> i32 {
-        self.info().comp_pstride(component as u8)
-    }
-
-    pub fn comp_stride(&self, component: u32) -> i32 {
-        self.info().comp_stride(component as u8)
-    }
-
-    pub fn comp_plane(&self, component: u32) -> u32 {
-        self.info().comp_plane(component as u8)
-    }
-
     pub fn plane_data(&self, plane: u32) -> Result<&[u8], glib::BoolError> {
         if plane >= self.n_planes() {
             return Err(glib::bool_error!(
@@ -661,37 +654,40 @@ impl<T> VideoFrameRef<T> {
         }
     }
 
-    pub fn as_ptr(&self) -> *const ffi::GstVideoFrame {
-        &self.frame
+    pub fn planes_data(&self) -> [&[u8]; 4] {
+        let mut planes = [[].as_slice(); 4];
+
+        for plane in 0..self.n_planes() {
+            planes[plane as usize] = self.plane_data(plane).unwrap();
+        }
+
+        planes
     }
 }
 
 impl<'a> VideoFrameRef<&'a gst::BufferRef> {
+    #[inline]
     pub unsafe fn from_glib_borrow(frame: *const ffi::GstVideoFrame) -> Borrowed<Self> {
-        assert!(!frame.is_null());
+        debug_assert!(!frame.is_null());
 
         let frame = ptr::read(frame);
-        let info = crate::VideoInfo(ptr::read(&frame.info));
-        let buffer = gst::BufferRef::from_ptr(frame.buffer);
         Borrowed::new(Self {
             frame,
-            buffer: Some(buffer),
-            info,
             unmap: false,
+            phantom: PhantomData,
         })
     }
 
+    #[inline]
     pub unsafe fn from_glib_full(frame: ffi::GstVideoFrame) -> Self {
-        let info = crate::VideoInfo(ptr::read(&frame.info));
-        let buffer = gst::BufferRef::from_ptr(frame.buffer);
         Self {
             frame,
-            buffer: Some(buffer),
-            info,
             unmap: true,
+            phantom: PhantomData,
         }
     }
 
+    #[inline]
     pub fn from_buffer_ref_readable<'b>(
         buffer: &'a gst::BufferRef,
         info: &'b crate::VideoInfo,
@@ -701,7 +697,7 @@ impl<'a> VideoFrameRef<&'a gst::BufferRef> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -713,17 +709,16 @@ impl<'a> VideoFrameRef<&'a gst::BufferRef> {
                 Err(glib::bool_error!("Failed to map VideoFrame"))
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
                     unmap: true,
+                    phantom: PhantomData,
                 })
             }
         }
     }
 
+    #[inline]
     pub fn from_buffer_ref_id_readable<'b>(
         buffer: &'a gst::BufferRef,
         id: i32,
@@ -734,7 +729,7 @@ impl<'a> VideoFrameRef<&'a gst::BufferRef> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map_id(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -747,48 +742,39 @@ impl<'a> VideoFrameRef<&'a gst::BufferRef> {
                 Err(glib::bool_error!("Failed to map VideoFrame"))
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
                     unmap: true,
+                    phantom: PhantomData,
                 })
             }
         }
     }
-
-    pub fn buffer(&self) -> &gst::BufferRef {
-        self.buffer.as_ref().unwrap()
-    }
 }
 
 impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
+    #[inline]
     pub unsafe fn from_glib_borrow_mut(frame: *mut ffi::GstVideoFrame) -> Self {
-        assert!(!frame.is_null());
+        debug_assert!(!frame.is_null());
 
         let frame = ptr::read(frame);
-        let info = crate::VideoInfo(ptr::read(&frame.info));
-        let buffer = gst::BufferRef::from_mut_ptr(frame.buffer);
         Self {
             frame,
-            buffer: Some(buffer),
-            info,
             unmap: false,
+            phantom: PhantomData,
         }
     }
 
+    #[inline]
     pub unsafe fn from_glib_full_mut(frame: ffi::GstVideoFrame) -> Self {
-        let info = crate::VideoInfo(ptr::read(&frame.info));
-        let buffer = gst::BufferRef::from_mut_ptr(frame.buffer);
         Self {
             frame,
-            buffer: Some(buffer),
-            info,
             unmap: true,
+            phantom: PhantomData,
         }
     }
 
+    #[inline]
     pub fn from_buffer_ref_writable<'b>(
         buffer: &'a mut gst::BufferRef,
         info: &'b crate::VideoInfo,
@@ -798,7 +784,7 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -812,17 +798,16 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
                 Err(glib::bool_error!("Failed to map VideoFrame"))
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
                     unmap: true,
+                    phantom: PhantomData,
                 })
             }
         }
     }
 
+    #[inline]
     pub fn from_buffer_ref_id_writable<'b>(
         buffer: &'a mut gst::BufferRef,
         id: i32,
@@ -833,7 +818,7 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
         assert!(info.is_valid());
 
         unsafe {
-            let mut frame = mem::MaybeUninit::zeroed();
+            let mut frame = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_video_frame_map_id(
                 frame.as_mut_ptr(),
                 info.to_glib_none().0 as *mut _,
@@ -848,21 +833,15 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
                 Err(glib::bool_error!("Failed to map VideoFrame"))
             } else {
                 let frame = frame.assume_init();
-                let info = crate::VideoInfo(ptr::read(&frame.info));
                 Ok(Self {
                     frame,
-                    buffer: Some(buffer),
-                    info,
                     unmap: true,
+                    phantom: PhantomData,
                 })
             }
         }
     }
 
-    pub fn buffer_mut(&mut self) -> &mut gst::BufferRef {
-        self.buffer.as_mut().unwrap()
-    }
-
     pub fn comp_data_mut(&mut self, component: u32) -> Result<&mut [u8], glib::BoolError> {
         let poffset = self.info().comp_poffset(component as u8) as usize;
         Ok(&mut self.plane_data_mut(self.format_info().plane()[component as usize])?[poffset..])
@@ -904,6 +883,25 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
         }
     }
 
+    pub fn planes_data_mut(&mut self) -> [&mut [u8]; 4] {
+        unsafe {
+            let mut planes = [
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+                [].as_mut_slice(),
+            ];
+
+            for plane in 0..self.n_planes() {
+                let slice = self.plane_data_mut(plane).unwrap();
+                planes[plane as usize] = slice::from_raw_parts_mut(slice.as_mut_ptr(), slice.len());
+            }
+
+            planes
+        }
+    }
+
+    #[inline]
     pub fn as_mut_ptr(&mut self) -> *mut ffi::GstVideoFrame {
         &mut self.frame
     }
@@ -912,6 +910,7 @@ impl<'a> VideoFrameRef<&'a mut gst::BufferRef> {
 impl<'a> ops::Deref for VideoFrameRef<&'a mut gst::BufferRef> {
     type Target = VideoFrameRef<&'a gst::BufferRef>;
 
+    #[inline]
     fn deref(&self) -> &Self::Target {
         unsafe { &*(self as *const Self as *const Self::Target) }
     }
@@ -921,6 +920,7 @@ unsafe impl<T> Send for VideoFrameRef<T> {}
 unsafe impl<T> Sync for VideoFrameRef<T> {}
 
 impl<T> Drop for VideoFrameRef<T> {
+    #[inline]
     fn drop(&mut self) {
         unsafe {
             if self.unmap {
@@ -938,6 +938,7 @@ pub trait VideoBufferExt {
 }
 
 impl VideoBufferExt for gst::BufferRef {
+    #[inline]
     fn video_flags(&self) -> crate::VideoBufferFlags {
         unsafe {
             let ptr = self.as_mut_ptr();
@@ -945,6 +946,7 @@ impl VideoBufferExt for gst::BufferRef {
         }
     }
 
+    #[inline]
     fn set_video_flags(&mut self, flags: crate::VideoBufferFlags) {
         unsafe {
             let ptr = self.as_mut_ptr();
@@ -952,6 +954,7 @@ impl VideoBufferExt for gst::BufferRef {
         }
     }
 
+    #[inline]
     fn unset_video_flags(&mut self, flags: crate::VideoBufferFlags) {
         unsafe {
             let ptr = self.as_mut_ptr();
