@@ -2,7 +2,7 @@
 
 use std::{fmt, ptr};
 
-use glib::translate::{from_glib, from_glib_none, FromGlib, IntoGlib, IntoGlibPtr, ToGlibPtr};
+use glib::translate::*;
 use gst::prelude::*;
 
 #[repr(transparent)]
@@ -159,8 +159,8 @@ impl VideoMeta {
         &self.0.stride[0..(self.0.n_planes as usize)]
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     #[doc(alias = "get_alignment")]
     #[inline]
     pub fn alignment(&self) -> crate::VideoAlignment {
@@ -173,8 +173,8 @@ impl VideoMeta {
         )
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     #[doc(alias = "get_plane_size")]
     #[doc(alias = "gst_video_meta_get_plane_size")]
     pub fn plane_size(&self) -> Result<[usize; crate::VIDEO_MAX_PLANES], glib::BoolError> {
@@ -182,10 +182,7 @@ impl VideoMeta {
 
         unsafe {
             glib::result_from_gboolean!(
-                ffi::gst_video_meta_get_plane_size(
-                    &self.0 as *const _ as usize as *mut _,
-                    &mut plane_size,
-                ),
+                ffi::gst_video_meta_get_plane_size(mut_override(&self.0), &mut plane_size,),
                 "Failed to get plane size"
             )?;
         }
@@ -193,8 +190,8 @@ impl VideoMeta {
         Ok(plane_size)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     #[doc(alias = "get_plane_height")]
     #[doc(alias = "gst_video_meta_get_plane_height")]
     pub fn plane_height(&self) -> Result<[u32; crate::VIDEO_MAX_PLANES], glib::BoolError> {
@@ -202,10 +199,7 @@ impl VideoMeta {
 
         unsafe {
             glib::result_from_gboolean!(
-                ffi::gst_video_meta_get_plane_height(
-                    &self.0 as *const _ as usize as *mut _,
-                    &mut plane_height,
-                ),
+                ffi::gst_video_meta_get_plane_height(mut_override(&self.0), &mut plane_height,),
                 "Failed to get plane height"
             )?;
         }
@@ -213,8 +207,8 @@ impl VideoMeta {
         Ok(plane_height)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     #[doc(alias = "gst_video_meta_set_alignment")]
     pub fn set_alignment(
         &mut self,
@@ -599,21 +593,21 @@ impl fmt::Debug for VideoOverlayCompositionMeta {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 #[repr(transparent)]
 #[doc(alias = "GstVideoCaptionMeta")]
 pub struct VideoCaptionMeta(ffi::GstVideoCaptionMeta);
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 unsafe impl Send for VideoCaptionMeta {}
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 unsafe impl Sync for VideoCaptionMeta {}
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 impl VideoCaptionMeta {
     #[doc(alias = "gst_buffer_add_video_caption_meta")]
     pub fn add<'a>(
@@ -655,8 +649,8 @@ impl VideoCaptionMeta {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 unsafe impl MetaAPI for VideoCaptionMeta {
     type GstType = ffi::GstVideoCaptionMeta;
 
@@ -667,8 +661,8 @@ unsafe impl MetaAPI for VideoCaptionMeta {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 impl fmt::Debug for VideoCaptionMeta {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("VideoCaptionMeta")
@@ -678,21 +672,21 @@ impl fmt::Debug for VideoCaptionMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 #[repr(transparent)]
 #[doc(alias = "GstVideoAFDMeta")]
 pub struct VideoAFDMeta(ffi::GstVideoAFDMeta);
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl Send for VideoAFDMeta {}
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl Sync for VideoAFDMeta {}
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 impl VideoAFDMeta {
     #[doc(alias = "gst_buffer_add_video_afd_meta")]
     pub fn add(
@@ -734,8 +728,8 @@ impl VideoAFDMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl MetaAPI for VideoAFDMeta {
     type GstType = ffi::GstVideoAFDMeta;
 
@@ -746,8 +740,8 @@ unsafe impl MetaAPI for VideoAFDMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 impl fmt::Debug for VideoAFDMeta {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("VideoAFDMeta")
@@ -758,21 +752,21 @@ impl fmt::Debug for VideoAFDMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 #[repr(transparent)]
 #[doc(alias = "GstVideoBarMeta")]
 pub struct VideoBarMeta(ffi::GstVideoBarMeta);
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl Send for VideoBarMeta {}
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl Sync for VideoBarMeta {}
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 impl VideoBarMeta {
     #[doc(alias = "gst_buffer_add_video_bar_meta")]
     pub fn add(
@@ -821,8 +815,8 @@ impl VideoBarMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe impl MetaAPI for VideoBarMeta {
     type GstType = ffi::GstVideoBarMeta;
 
@@ -833,8 +827,8 @@ unsafe impl MetaAPI for VideoBarMeta {
     }
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 impl fmt::Debug for VideoBarMeta {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("VideoBarMeta")
@@ -846,21 +840,21 @@ impl fmt::Debug for VideoBarMeta {
     }
 }
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 #[repr(transparent)]
 #[doc(alias = "GstVideoCodecAlphaMeta")]
 pub struct VideoCodecAlphaMeta(ffi::GstVideoCodecAlphaMeta);
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 unsafe impl Send for VideoCodecAlphaMeta {}
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 unsafe impl Sync for VideoCodecAlphaMeta {}
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 impl VideoCodecAlphaMeta {
     #[doc(alias = "gst_buffer_add_video_codec_alpha_meta")]
     pub fn add(
@@ -889,8 +883,8 @@ impl VideoCodecAlphaMeta {
     }
 }
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 unsafe impl MetaAPI for VideoCodecAlphaMeta {
     type GstType = ffi::GstVideoCodecAlphaMeta;
 
@@ -901,8 +895,8 @@ unsafe impl MetaAPI for VideoCodecAlphaMeta {
     }
 }
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 impl fmt::Debug for VideoCodecAlphaMeta {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("VideoCodecAlphaMeta")
@@ -911,6 +905,262 @@ impl fmt::Debug for VideoCodecAlphaMeta {
     }
 }
 
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+#[repr(transparent)]
+#[doc(alias = "GstVideoSEIUserDataUnregisteredMeta")]
+pub struct VideoSeiUserDataUnregisteredMeta(ffi::GstVideoSEIUserDataUnregisteredMeta);
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+unsafe impl Send for VideoSeiUserDataUnregisteredMeta {}
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+unsafe impl Sync for VideoSeiUserDataUnregisteredMeta {}
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+impl VideoSeiUserDataUnregisteredMeta {
+    #[doc(alias = "gst_buffer_add_video_sei_user_data_unregistered_meta")]
+    pub fn add<'a>(
+        buffer: &'a mut gst::BufferRef,
+        uuid: &[u8; 16],
+        data: &[u8],
+    ) -> gst::MetaRefMut<'a, Self, gst::meta::Standalone> {
+        skip_assert_initialized!();
+        assert!(!data.is_empty());
+        unsafe {
+            let meta = ffi::gst_buffer_add_video_sei_user_data_unregistered_meta(
+                buffer.as_mut_ptr(),
+                mut_override(uuid.as_ptr()),
+                mut_override(data.as_ptr()),
+                data.len(),
+            );
+
+            Self::from_mut_ptr(buffer, meta)
+        }
+    }
+
+    #[doc(alias = "get_data")]
+    #[inline]
+    pub fn data(&self) -> &[u8] {
+        if self.0.size == 0 {
+            return &[];
+        }
+        // SAFETY: In the C API we have a pointer data and a size variable
+        // indicating the length of the data. Here we convert it to a size,
+        // making sure we read the size specified in the C API.
+        unsafe {
+            use std::slice;
+            slice::from_raw_parts(self.0.data, self.0.size)
+        }
+    }
+
+    #[doc(alias = "get_uuid")]
+    #[inline]
+    pub fn uuid(&self) -> [u8; 16] {
+        self.0.uuid
+    }
+}
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+impl fmt::Debug for VideoSeiUserDataUnregisteredMeta {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("VideoSeiUserDataUnregisteredMeta")
+            .field(
+                "uuid",
+                &format!("0x{:032X}", u128::from_be_bytes(self.uuid())),
+            )
+            .field("data", &self.data())
+            .finish()
+    }
+}
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
+unsafe impl MetaAPI for VideoSeiUserDataUnregisteredMeta {
+    type GstType = ffi::GstVideoSEIUserDataUnregisteredMeta;
+
+    #[doc(alias = "gst_video_sei_user_data_unregistered_meta_api_get_type")]
+    fn meta_api() -> glib::Type {
+        unsafe {
+            glib::translate::from_glib(ffi::gst_video_sei_user_data_unregistered_meta_api_get_type())
+        }
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+#[repr(transparent)]
+#[doc(alias = "GstAncillaryMeta")]
+pub struct AncillaryMeta(ffi::GstAncillaryMeta);
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+unsafe impl Send for AncillaryMeta {}
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+unsafe impl Sync for AncillaryMeta {}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl AncillaryMeta {
+    #[doc(alias = "gst_buffer_add_ancillary_meta")]
+    pub fn add(buffer: &mut gst::BufferRef) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+        skip_assert_initialized!();
+        unsafe {
+            let meta = ffi::gst_buffer_add_ancillary_meta(buffer.as_mut_ptr());
+
+            Self::from_mut_ptr(buffer, meta)
+        }
+    }
+
+    #[inline]
+    pub fn field(&self) -> crate::AncillaryMetaField {
+        unsafe { from_glib(self.0.field) }
+    }
+
+    #[inline]
+    pub fn set_field(&mut self, field: crate::AncillaryMetaField) {
+        self.0.field = field.into_glib();
+    }
+
+    #[inline]
+    pub fn c_not_y_channel(&self) -> bool {
+        unsafe { from_glib(self.0.c_not_y_channel) }
+    }
+
+    #[inline]
+    pub fn set_c_not_y_channel(&mut self, c_not_y_channel: bool) {
+        self.0.c_not_y_channel = c_not_y_channel.into_glib();
+    }
+
+    #[inline]
+    pub fn line(&self) -> u16 {
+        self.0.line
+    }
+
+    #[inline]
+    pub fn set_line(&mut self, line: u16) {
+        self.0.line = line;
+    }
+
+    #[inline]
+    pub fn offset(&self) -> u16 {
+        self.0.offset
+    }
+
+    #[inline]
+    pub fn set_offset(&mut self, offset: u16) {
+        self.0.offset = offset;
+    }
+
+    #[inline]
+    pub fn did(&self) -> u16 {
+        self.0.DID
+    }
+
+    #[inline]
+    pub fn set_did(&mut self, did: u16) {
+        self.0.DID = did;
+    }
+
+    #[inline]
+    pub fn sdid_block_number(&self) -> u16 {
+        self.0.SDID_block_number
+    }
+
+    #[inline]
+    pub fn set_sdid_block_number(&mut self, sdid_block_number: u16) {
+        self.0.SDID_block_number = sdid_block_number;
+    }
+
+    #[inline]
+    pub fn data_count(&self) -> u16 {
+        self.0.data_count
+    }
+
+    #[inline]
+    pub fn checksum(&self) -> u16 {
+        self.0.checksum
+    }
+
+    #[inline]
+    pub fn set_checksum(&mut self, checksum: u16) {
+        self.0.checksum = checksum;
+    }
+
+    #[inline]
+    pub fn data(&self) -> &[u16] {
+        if self.0.data_count & 0xff == 0 {
+            return &[];
+        }
+        unsafe {
+            use std::slice;
+
+            slice::from_raw_parts(self.0.data, (self.0.data_count & 0xff) as usize)
+        }
+    }
+
+    #[inline]
+    pub fn data_mut(&mut self) -> &mut [u16] {
+        if self.0.data_count & 0xff == 0 {
+            return &mut [];
+        }
+        unsafe {
+            use std::slice;
+
+            slice::from_raw_parts_mut(self.0.data, (self.0.data_count & 0xff) as usize)
+        }
+    }
+
+    #[inline]
+    pub fn set_data(&mut self, data: glib::Slice<u16>) {
+        unsafe {
+            assert!(data.len() < 256);
+            self.0.data_count = data.len() as u16;
+            self.0.data = data.into_glib_ptr();
+        }
+    }
+
+    #[inline]
+    pub fn set_data_count_upper_two_bits(&mut self, upper_two_bits: u8) {
+        assert!(upper_two_bits & !0x03 == 0);
+        self.0.data_count = ((upper_two_bits as u16) << 8) | self.0.data_count & 0xff;
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+unsafe impl MetaAPI for AncillaryMeta {
+    type GstType = ffi::GstAncillaryMeta;
+
+    #[doc(alias = "gst_ancillary_meta_api_get_type")]
+    #[inline]
+    fn meta_api() -> glib::Type {
+        unsafe { from_glib(ffi::gst_ancillary_meta_api_get_type()) }
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl fmt::Debug for AncillaryMeta {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("AncillaryMeta")
+            .field("field", &self.field())
+            .field("c_not_y_channel", &self.c_not_y_channel())
+            .field("line", &self.line())
+            .field("offset", &self.offset())
+            .field("did", &self.did())
+            .field("sdid_block_number", &self.sdid_block_number())
+            .field("data_count", &self.data_count())
+            .field("data", &self.data())
+            .field("checksum", &self.checksum())
+            .finish()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1062,4 +1312,46 @@ mod tests {
         assert_eq!(meta.plane_size().unwrap(), [79360, 39680, 0, 0]);
         assert_eq!(meta.plane_height().unwrap(), [248, 124, 0, 0]);
     }
+
+    #[test]
+    #[cfg(feature = "v1_22")]
+    fn test_get_video_sei_user_data_unregistered_meta() {
+        gst::init().unwrap();
+
+        const META_UUID: &[u8; 16] = &[
+            0x4D, 0x49, 0x53, 0x50, 0x6D, 0x69, 0x63, 0x72, 0x6F, 0x73, 0x65, 0x63, 0x74, 0x69,
+            0x6D, 0x65,
+        ];
+
+        const META_DATA: &[u8] = &[
+            0x1f, 0x00, 0x05, 0xff, 0x21, 0x7e, 0xff, 0x29, 0xb5, 0xff, 0xdc, 0x13,
+        ];
+
+        let buffer_data = &[
+            &[0x00, 0x00, 0x00, 0x20, 0x06, 0x05, 0x1c],
+            META_UUID as &[u8],
+            META_DATA,
+            &[
+                0x80, 0x00, 0x00, 0x00, 0x14, 0x65, 0x88, 0x84, 0x00, 0x10, 0xff, 0xfe, 0xf6, 0xf0,
+                0xfe, 0x05, 0x36, 0x56, 0x04, 0x50, 0x96, 0x7b, 0x3f, 0x53, 0xe1,
+            ],
+        ]
+        .concat();
+
+        let mut harness = gst_check::Harness::new("h264parse");
+        harness.set_src_caps_str(r#"
+            video/x-h264, stream-format=(string)avc,
+            width=(int)1920, height=(int)1080, framerate=(fraction)25/1,
+            bit-depth-chroma=(uint)8, parsed=(boolean)true,
+            alignment=(string)au, profile=(string)high, level=(string)4,
+            codec_data=(buffer)01640028ffe1001a67640028acb200f0044fcb080000030008000003019478c1924001000568ebccb22c
+        "#);
+        let buffer = gst::Buffer::from_slice(buffer_data.clone());
+        let buffer = harness.push_and_pull(buffer).unwrap();
+
+        let meta = buffer.meta::<VideoSeiUserDataUnregisteredMeta>().unwrap();
+        assert_eq!(meta.uuid(), *META_UUID);
+        assert_eq!(meta.data(), META_DATA);
+        assert_eq!(meta.data().len(), META_DATA.len());
+    }
 }
